<?php

namespace App\Service;

use Symfony\Component\Process\Process;

readonly class BackupService
{
    private const COMMAND_DUMP = 'mariadb-dump';
    private const COMMAND_DB = 'mariadb';

    public function __construct(
        private DatabaseCredentials $credentials
    ) {}

    public function backup(string $db, ?string $fileName = null): string
    {
        if (!$fileName) {
            $date = date('y_m_d_H_i');
            $fileName = sprintf('%s_%s_backup.sql', $db, $date);
        }

        $process = new Process([
            self::COMMAND_DUMP,
            '-u', $this->credentials->user,
            $db,
        ]);
        $process->setEnv(['MYSQL_PWD' => $this->credentials->password]);
        $process->start();

        $stream = fopen($fileName, 'w+');
        foreach ($process as $type => $data) {
            if ($process::OUT === $type) {
                fwrite($stream, $data);
            } else { // $process::ERR === $type
                echo "\nError: " . $data;
            }
        }

        return $fileName;
    }

    public function restore(string $db, string $fileName): void
    {
        $process = new Process([
            self::COMMAND_DB,
            '-u', $this->credentials->user,
            $db,
        ]);

        $stream = fopen($fileName, 'r');

        $process->setEnv(['MYSQL_PWD' => $this->credentials->password]);
        $process->setInput($stream);
        $process->run();
    }
}